Attribute VB_Name = "modOpenSaveDialog"
' Wave file steganography packer/unpacker.
' modOpenSaveDialog.bas - module for the work with files/folders dialogs
'  Krivous Anatolii Anatolevich (The trick), 2015

Option Explicit

Private Type OPENFILENAME
    lStructSize         As Long
    hWndOwner           As Long
    hInstance           As Long
    lpstrFilter         As String
    lpstrCustomFilter   As String
    nMaxCustFilter      As Long
    nFilterIndex        As Long
    lpstrFile           As String
    nMaxFile            As Long
    lpstrFileTitle      As String
    nMaxFileTitle       As Long
    lpstrInitialDir     As String
    lpstrTitle          As String
    Flags               As Long
    nFileOffset         As Integer
    nFileExtension      As Integer
    lpstrDefExt         As String
    lCustData           As Long
    lpfnHook            As Long
    lpTemplateName      As String
End Type

Private Type BrowseInfo
    hWndOwner           As Long
    pIDLRoot            As Long
    pszDisplayName      As String
    lpszTitle           As String
    ulFlags             As Long
    lpfnCallback        As Long
    lParam              As Long
    iImage              As Long
End Type

Private Declare Function GetOpenFileName _
                         Lib "comdlg32.dll" _
                         Alias "GetOpenFileNameA" ( _
                         ByRef pOpenFilename As OPENFILENAME) As Long
Private Declare Function CoTaskMemFree _
                         Lib "ole32.dll" ( _
                         ByVal hMem As Long) As Long
Private Declare Function SHBrowseForFolder _
                         Lib "shell32" ( _
                         ByRef lpbi As BrowseInfo) As Long
Private Declare Function SHGetPathFromIDList _
                         Lib "shell32" ( _
                         ByVal pidList As Long, _
                         ByVal lpBuffer As String) As Long

Private Const x_MaxBuffer           As Long = 32768
Private Const OFN_ENABLESIZING      As Long = &H800000
Private Const OFN_EXPLORER          As Long = &H80000
Private Const BIF_EDITBOX           As Long = &H10
Private Const BIF_NEWDIALOGSTYLE    As Long = &H40
Private Const BIF_USENEWUI          As Long = (BIF_NEWDIALOGSTYLE Or BIF_EDITBOX)

' // Show file open dialog and get name of selected file
Public Function ShowOpen( _
                ByVal Hwnd As Long, _
                ByRef Filter As String) As String
    Dim N           As Long
    Dim O           As Long
    Dim FileStruct  As OPENFILENAME
    
    With FileStruct
    
        .hWndOwner = Hwnd
        .lpstrFile = String(x_MaxBuffer, 0)
        .nMaxFile = x_MaxBuffer - 1
        .lpstrFileTitle = .lpstrFile
        .nMaxFileTitle = x_MaxBuffer - 1
        .Flags = OFN_ENABLESIZING Or OFN_EXPLORER
        .lStructSize = Len(FileStruct)
        .lpstrFilter = Filter
        O = GetOpenFileName(FileStruct)
        
        If O Then
            N = InStr(1, .lpstrFile, vbNullChar)
            ShowOpen = Left$(.lpstrFile, N - 1)
        End If
        
    End With
    
End Function

' // Show browse folder dialog and get selected path
Public Function BrowseFolder( _
                ByVal Hwnd As Long) As String
    Dim iNull       As Integer
    Dim lpIDList    As Long
    Dim lResult     As Long
    Dim BI          As BrowseInfo
    
    BI.hWndOwner = Hwnd
    BI.ulFlags = BIF_USENEWUI
    lpIDList = SHBrowseForFolder(BI)
    
    If lpIDList Then
    
        BrowseFolder = String$(x_MaxBuffer, 0)
        SHGetPathFromIDList lpIDList, BrowseFolder
        Call CoTaskMemFree(lpIDList)
        iNull = InStr(BrowseFolder, vbNullChar)
        If iNull Then BrowseFolder = Left$(BrowseFolder, iNull - 1)
        
    End If
    
End Function
