Attribute VB_Name = "modWaveFile"
' Wave file steganography packer/unpacker.
' modWaveFile.bas - module for loading WAVE-files
'  Krivous Anatolii Anatolevich (The trick), 2015

Option Explicit

Private Type MMCKINFO
    ckid            As Long
    ckSize          As Long
    fccType         As Long
    dwDataOffset    As Long
    dwFlags         As Long
End Type
Public Type WAVEFORMAT
    wFormatTag      As Integer
    nChannels       As Integer
    nSamplesPerSec  As Long
    nAvgBytesPerSec As Long
    nBlockAlign     As Integer
    wBitsPerSample  As Integer
    cbSize          As Integer
End Type

Private Declare Function mmioClose _
                         Lib "winmm.dll" ( _
                         ByVal hmmio As Long, _
                         ByVal uFlags As Long) As Long
Private Declare Function mmioDescend _
                         Lib "winmm.dll" ( _
                         ByVal hmmio As Long, _
                         ByRef lpck As MMCKINFO, _
                         ByRef lpckParent As Any, _
                         ByVal uFlags As Long) As Long
Private Declare Function mmioOpen _
                         Lib "winmm.dll" _
                         Alias "mmioOpenA" ( _
                         ByVal szFileName As String, _
                         ByRef lpmmioinfo As Any, _
                         ByVal dwOpenFlags As Long) As Long
Private Declare Function mmioRead _
                         Lib "winmm.dll" ( _
                         ByVal hmmio As Long, _
                         ByRef pch As Any, _
                         ByVal cch As Long) As Long
Private Declare Function mmioStringToFOURCC _
                         Lib "winmm.dll" _
                         Alias "mmioStringToFOURCCA" ( _
                         ByVal sz As String, _
                         ByVal uFlags As Long) As Long
Private Declare Function mmioAscend _
                         Lib "winmm.dll" ( _
                         ByVal hmmio As Long, _
                         ByRef lpck As MMCKINFO, _
                         ByVal uFlags As Long) As Long

Private Const MMIO_READ         As Long = &H0
Private Const MMIO_FINDCHUNK    As Long = &H10
Private Const MMIO_FINDRIFF     As Long = &H20

Public Format       As WAVEFORMAT             ' // Audio format
Public WaveData()   As Integer                ' // Audio data
Public Offset       As Long                   ' // Offset to audion data from beggining of the file

' // Load wave file
Public Function ReadWaveFile( _
                ByRef FileName As String) As Boolean
    Dim hIn     As Long
    Dim ret     As Long
    Dim Inf     As MMCKINFO
    Dim sInf    As MMCKINFO
    Dim ct  As Long
    ' // Try to open file
    hIn = mmioOpen(FileName, ByVal 0, MMIO_READ)
    If (hIn = 0) Then MsgBox "Error opening the file": Exit Function
    ' // Check whether is wave-file or not
    Inf.fccType = mmioStringToFOURCC("WAVE", 0)
    If mmioDescend(hIn, Inf, ByVal 0, MMIO_FINDRIFF) Then _
        mmioClose hIn, 0: MsgBox "It isn't the WAVE-file": Exit Function
    ' // Find format-chunk
    sInf.ckid = mmioStringToFOURCC("fmt", 0)
    If mmioDescend(hIn, sInf, Inf, MMIO_FINDCHUNK) Then _
        mmioClose hIn, 0: MsgBox "Format chunk not found": Exit Function
    ' // Check size of format
    If sInf.ckSize > Len(Format) Then _
        mmioClose hIn, 0: MsgBox "Unsupported format": Exit Function
    ' // Read format
    If mmioRead(hIn, Format, sInf.ckSize) = -1 Then _
        mmioClose hIn, 0: MsgBox "Unable to read format": Exit Function
        
    mmioAscend hIn, sInf, 0
    ' // Check bitness
    If Format.wBitsPerSample <> 16 Then _
        mmioClose hIn, 0: MsgBox "Unsupported bitness": Exit Function
    ' // Find data chunk
    sInf.ckid = mmioStringToFOURCC("data", 0)
    If mmioDescend(hIn, sInf, Inf, MMIO_FINDCHUNK) Then _
        mmioClose hIn, 0: MsgBox "Audio data not found": Exit Function
    ' // Get number of samples
    ct = sInf.ckSize \ 2
    If ct = 0 Then mmioClose hIn, 0: MsgBox "Zero file size": Exit Function
    ' // Alloc buffer
    ReDim WaveData(ct - 1)
    ' // Save offset
    Offset = sInf.dwDataOffset
    ' // Read data
    If mmioRead(hIn, WaveData(0), sInf.ckSize) = -1 Then _
        mmioClose hIn, 0: MsgBox "Unable to read audio data": Exit Function
        
    mmioClose hIn, 0
    
    ReadWaveFile = True
    
End Function
